"""
    News Desk Product
    ==== ==== =======

Adds a new command 'dtml-news' to download news and then render a block
with the returned data. Supports 'dtml-else' to catch errors cleanly.

Functions like 'dtml-in', in that the returned data is added to the
environment. The following will exist, though individual data-parsers
may add others:
    url         # The link for this item
    headline    # The Headline for this item
    body        # The teaser for this item
"""
# $Id: NewsTag.py,v 1.2 2001/10/21 13:32:42 adrian Exp $
#
# $Log: NewsTag.py,v $
# Revision 1.2  2001/10/21 13:32:42  adrian
# no message
#
#
__version__ = '$Revision: 1.2 $'[11:-2]

from DocumentTemplate.DT_Util import *
from DocumentTemplate.DT_String import String
import Parsers

class NewsTag:
    """ dtml-news tag """
    blockContinuations=('news', 'else',)
    name='news'

    def __init__(self, blocks):
        """ Verify parameters etc... """
        self.sections = []
        self.elses = None
# Get block data.
        for b in blocks:
            tname, args, section = b
            if tname == 'news':
                type   = ''
                server = None
                source = None
                sample = 0
                debug = 0
                args = parse_params(args, type = '',
                    server = None, noserver = 1, source = None, sample = 1, debug = 1)
                if args.has_key('type'):
                    type = args['type']
                if args.has_key('server'):
                    server = args['server']
                if args.has_key('noserver'):
                    server = ''
                if args.has_key('source'):
                    source = args['source']
                if args.has_key('sample'):
                    sample = 1
                if args.has_key('debug'):
                    debug = 1
                try:
                    parser = Parsers.ParserList[type]
                except:
                    raise ParseError, ("type must be one of '%s'" %
                        join(Parsers.ParserList.keys(), "', '"), "news")
                #try:
                if 1:
                    self.sections.append(
                        parser(server, source, sample),
                        section.blocks,
                        debug,
                    )
                #except:
                #    raise ParseError, ("Error creating '%s' parser" %
                #        type, "news")
            elif tname == 'else':
                if self.elses:
                    raise ParseError, ("too many else blocks", "news")
                else:
                    self.elses = section.blocks

    def render(self, md):
        """
            Render the sections in order.
            If all sections are empty then render the ELSE if there is one.
        """
        results=[]
        for pb in self.sections:
            parser, blocks, debug = pb
            parser._internal_reset()
            parser.run(md)
            if debug:
                results.append(parser.debug())
            else:
                for item in parser.Stories:
                    item['sequence-item'] = item
                    md._push(item)
                    blk = render_blocks(blocks, md)
                    results.append(blk)
                    md._pop()
        if not results:
            if self.elses:
                return render_blocks(self.elses, md)
        return join(results, '')

    __call__ = render

String.commands['news'] = NewsTag
