"""
    News Desk Product - BaseParser

This file contains the basic parser code that is shared by all news-sources.
It also contains the list of all registered parsers.
"""
# $Id: BaseParser.py,v 1.2 2001/10/21 13:35:00 adrian Exp $
#
# $Log: BaseParser.py,v $
# Revision 1.2  2001/10/21 13:35:00  adrian
# no message
#
#
__version__ = '$Revision: 1.2 $'[11:-2]

import os
import httplib
from Globals import package_home
from urlparse import urlparse

ParserList = {}

# This doesn't do much yet, but it could verify things about the parser
# Or perform a test run etc...
def registerParser(parser):
    """ Register a parser in the ParserList """
    ParserList[parser.name] = parser

class BaseParser:
    """ Mixin class containing basic parser functions """
    name = 'Incomplete'
    server = None
    sample = None
    sources = {}

    def __init__(self, server=None, source=None, sample=None):
        """ Init this parser and link it to a datasource
            server = None, '.' or address
                None    = Use default server (If any)
                ''      = Source is a local file
                address = Source is a remote file
            source = None, tag or path
                None    = Use default source (If any)
                tag     = Use the specified source
                path    = Use the specified file
            sample = 1 or 0 (Any true or Any false)
                1       = Ignore server/source and use sample
                0       = Use server/source
        """
        self._internal_reset()
        if sample:
            self.server = None
            self.file = None
        else:
            if server:
                self.server = server
            elif server==None:
                self.server = self.__class__.server
            else:
                self.server = None
            if source:
                if self.sources.has_key(source):
                    self.file = self.sources[source]
                else:
                    self.file = source
            else:
                if self.sources.has_key('default'):
                    self.file = self.sources['default']
                else:
                    raise ParameterError, "No default source specified for Parser '%s' and no default found" % self.name

    def _internal_reset(self):
        """ """
        if not self.sample or not hasattr(self, 'data'):
            self.data = ''
        self.Stories = []
        if hasattr(self, 'reset'):
            self.reset()

    def run(self, acquirer=None):
        """ Trigger parser to connect, download and
            parse the data from the select datasource """
        self._internal_reset()
        if acquirer==None:
            acquirer=self
#        raise "DEBUG:\n\n%s" % (, )
        if not self.file:
            path = os.path.split(package_home(globals()))[0]
            file = open(os.path.join(path, 'Samples', self.sample))
            self.feed(file.read())
            file.close()
        else:
            server = self.server
            file = self.file
            if not self.server:
                url = acquirer.getitem(self.file, 0).absolute_url()
                d1, server, file, d2, d3, d4 = urlparse(url)
            try:
                h = httplib.HTTP(server)
            except:
                raise "Failed to connect to server %s" % self.server, ""
            h.putrequest('GET', file)
            h.putheader('Accept', 'text/plain')
            h.putheader('Accept', 'text/html')
            h.putheader('Accept', 'text/*')
            h.endheaders()
            errcode, errmsg, headers = h.getreply()
            if errcode == 200:
                self.feed("%s" % h.getfile().read())
            elif errcode == 301:
                raise "HTTP error:%d, message:%s, url:[http://%s%s]<hr><pre>%s</pre>" % (errcode, errmsg, self.server, self.file, headers), ""
            else:
                raise "HTTP error:%d, message:%s, url:[http://%s%s]" % (errcode, errmsg, self.server, self.file), ""
        self.close()                                                          

    def feed(self, data):
        """ Internal Function
            This receives the data from the datasource """
        self.data = self.data + data

    def close(self):
        """ This should be overridden in your parser """
        self.Stories = [
            {
                'url'       : 'http://www.zope.org/Members/haqa',
                'headline'  : 'This parser does not work!',
                'body'      : 'This parser does not work, you should get a newer version!',
            }
        ]

    def debug(self):
        """ """
        html = "<ul>"
        for item in self.Stories:
            html = html + "<li>%s : (%s)<br/>\n%s</li>\n" % (item['headline'], item['url'], item['body'])
        html = html + "</ul>\n"
        return html
