"""
    News Desk Product - RSS Parser

File Format: Netscape RSS
"""
# $Id: RSS.py,v 1.2 2001/10/21 13:35:00 adrian Exp $
#
# $Log: RSS.py,v $
# Revision 1.2  2001/10/21 13:35:00  adrian
# no message
#
#
__version__ = '$Revision: 1.2 $'[11:-2]

import string
import xmllib
import BaseParser

class XMLNode:
    """ A really simple XML node class """
    def __init__(self, tag, attrs):
        """ Create this node """
        self.tag = tag
        self.attrs = attrs
        self.cdata = ''
        self.children = []
        self.parent = None

    def addCDATA(self, cdata):
        """ Add CDATA to this node """
        self.cdata = self.cdata + cdata

    def addChild(self, child):
        """ Add a node to the children list """
        self.children.append(child)
        child.parent = self

    def getNextChildNamed(self, name, start=None):
        """ Get the NEXT child called ... """
        found = start == None
        for item in self.children:
            if found and item.tag == name:
                return item
            if item == start:
                found = 1
        return None

    def dump(self, level=''):
        """ Dump this node and its children """
        data = "%s&lt;%s" % (level, self.tag, )
        for item in self.attrs.keys():
            data = data + ' %s="%s"' % (item, self.attrs[item], )
        data = data + "&gt;<br />\n"
        if self.cdata:
            data = data + "%s&nbsp;%s<br />\n" % (level, self.cdata, )
        for item in self.children:
            data = data + item.dump(level + '&nbsp;')
        data = data + "%s&lt;/%s&gt;<br />\n" % (level, self.tag, )
        return data

class RSS(BaseParser.BaseParser, xmllib.XMLParser):
    """ A NewsTag/NewsCatcher parser for any Netscape RSS format channel """
    name = 'RSS'
    sample = 'sample.rss'

    def __init__(self, server=None, source=None, sample=None):
        """ Call BaseParser.__init__ and XMLParser.__init__ """
        xmllib.XMLParser.__init__(self)
        BaseParser.BaseParser.__init__(self, server, source, sample)

    def _internal_reset(self):
        """ """
        BaseParser.BaseParser._internal_reset(self)
        self.document = []
        self.currentNode = None

    def feed(self, data):
        """ Feed the data to the XML parser instead of the BaseParser """
        data = string.strip(data)
        xmllib.XMLParser.feed(self, data)

    def close(self):
        """ Close the processor and process the received data """
        xmllib.XMLParser.close(self)
#        raise "Debug", "<br />%s" % self.document[0].dump()
        if len(self.document)!=1:
            raise NotImplementedError, "There are %d root objects in this XML file - There should only be one." % (len(self.document),)
        start = self.document[0].getNextChildNamed('channel')
#        raise "Debug", "<br />%s" % start.dump()
        item = start.getNextChildNamed('item')
        while item:
            line = {'headline': '', 'url': '', 'body': ''}
            for child in item.children:
                line[child.tag] = child.cdata
            if line.has_key('link'):
                line['url'] = line['link']
            if line.has_key('title'):
                line['headline'] = line['title']
            if line.has_key('description'):
                line['body'] = line['description']
            self.Stories.append(line)
            item = start.getNextChildNamed('item', item)

    def unknown_starttag(self, tag, attrs):
        """ """
        t = string.split(tag)
        if len(t)>1:
            tag = t[1]
            ns = t[0]
        node = XMLNode(tag, attrs)
        if self.currentNode:
            self.currentNode.addChild(node)
        else:
            self.document.append(node)
        self.currentNode = node

    def unknown_endtag(self, tag):
        """ """
        t = string.split(tag)
        if len(t)>1:
            tag = t[1]
            ns = t[0]
        self.currentNode = self.currentNode.parent

    def handle_data(self, data):
        """ Pass data to the cdata handler after whitespace stripping """
        self.handle_cdata(string.strip(data))

    def handle_cdata(self, data):
        """ """
        if self.currentNode:
            self.currentNode.addCDATA(data)

    def debug(self):
        """ """
        html = BaseParser.BaseParser.debug(self)
        html = html + "<hr>"
        html = html + self.document[0].dump()
        return html

BaseParser.registerParser(RSS)
